@@ -31,8 +31,12 @@
 struct output_buffer_t
 {
   output_buffer_t (option_parser_t *parser)
-		  : options (parser),
-		    format (parser) {}
+		  : options (parser,
+			     g_strjoinv ("/", (gchar**) hb_buffer_serialize_list_formats ())),
+		    format (parser),
+		    gs (NULL),
+		    line_no (0),
+		    font (NULL) {}
 
   void init (const font_options_t *font_opts)
   {
@@ -47,9 +51,13 @@ struct output_buffer_t
       output_format = hb_buffer_serialize_format_from_string (options.output_format, -1);
     if (!hb_buffer_serialize_format_to_string (output_format))
     {
-      fail (false, "Unknown output format `%s'; supported formats are: %s",
-	    options.output_format,
-	    g_strjoinv (" / ", (gchar**) hb_buffer_serialize_list_formats ()));
+      if (options.explicit_output_format)
+	fail (false, "Unknown output format `%s'; supported formats are: %s",
+	      options.output_format, options.supported_formats);
+      else
+	/* Just default to TEXT if not explicitly requested and the
+	 * file extension is not recognized. */
+	output_format = HB_BUFFER_SERIALIZE_FORMAT_TEXT;
     }
 
     unsigned int flags = HB_BUFFER_SERIALIZE_FLAGS_DEFAULT;
